local T = ...
local spickle_func = spickle2

T.DeleteRoids = false
T.DeleteRoidsList = {}

T.BGColors = {
	[0] = string.format("%s %d", ListColors[0], ListColors.Alpha),
	[1] = string.format("%s %d", ListColors[1], ListColors.Alpha),
	[2] = string.format("%s %d", ListColors[2], ListColors.SelectedAlpha),
	[3] = string.format("%s %d", "255 255 255", 255),
	[4] = string.format("%s %d", "56 150 56", 255),
	Numbers = {
		[0] = ListColors.Numbers[0],
		[1] = ListColors.Numbers[1],
		[2] = ListColors.Numbers[2],
		[3] = {255, 255, 255, 255},
		[4] = {255, 50, 50, 255},
	
	},
}

T.RoidNames = {
	["Ice Crystal"] = true,
	["Asteroid"] = true,
}

T.ShipsWithMineralScanners = {
	["Warthog Mineral Extractor"] = 500,
	["Behemoth Heaver Miner MkII"] = 1000,
	["Goliath"] = 500,
	["Trident Light Frigate"] = 500,
}

local ore_reference = {
	[1] = {
		'Aq',
		'Si',
		'Ca',
		'Fe',
		'Is',
		'Va',
		'Xi',
		'La',
		'De',
		'Py',
		'Ap',
		'Pe',
		'He',
		'Sa',
	},
    [2] = {
		Aq = 'Aquean',
		Si = 'Silicate',
		Ca = 'Carbonic',
		Fe = 'Ferric',
		Is = 'Ishik',
		Va = 'VanAzek',
		Xi = 'Xithricite',
		La = 'Lanthanic',
		De = 'Denic',
		Py = 'Pyronic',
		Ap = 'Apicene',
		Pe = 'Pentric',
		He = 'Heliocene',
		Sa = 'Sammic',
	},
    [3] = {
		Aquean = 'Aq',
		Silicate = 'Si',
		Carbonic = 'Ca',
		Ferric = 'Fe',
		Ishik = 'Is',
		VanAzek = 'Va',
		Xithricite = 'Xi',
		Lanthanic = 'La',
		Denic = 'De',
		Pyronic = 'Py',
		Apicene = 'Ap',
		Pentric = 'Pe',
		Heliocene = 'He',
		Sammic = 'Sa',
	}
}
for i, v in ipairs(ore_reference[1]) do ore_reference[1][v] = i end
T.ore_reference = ore_reference


-- Holes all of the roid data.  use T.GetData() to access
local asteroid_data

-- Holds calculated summaries for systems and sectors
local summary_data
function T.GetSummary()
	return summary_data
end


-- Holds sectors needing to be updated because of newly scanned roids.
-- Updates should only be processed when viewing the UI, not during active flight
local new_data = false -- flag to track new data for saving
do
	local count = 0
	T.pending = {
	--	[<sectorid>] = {
	-- 		[<roidID>] = true,
	-- 	},
		AddRoid = function(self, sectorid, roidID, roidtbl)
			self[sectorid] = self[sectorid] or {}
			self[sectorid][roidID] = roidtbl
			count = count + 1
			new_data = true
			if (not T.Save.timer:IsActive()) then
				T.Save.timer:SetTimeout(T.Save.interval, function() T.SaveData(GetSystemID(sectorid)) end)
			end
		end,

		Clear = function(self, sectorid)
			if (sectorid) then
				for i in pairs(self[sectorid] or {}) do
					count = count - 1
				end
				self[sectorid] = nil
			else
				for sectorid in pairs(self.pending) do
					if (type(sectorid) == "number") then
						self[sectorid] = nil
					end
				end
				count = 0
			end
		end,

		GetCount = function(sectorid)
			if (sectorid) then
				local sector_count = 0
				for i in pairs(self.pending[sectorid]) do
					sector_count = sector_count + 1
				end
				return sector_count
			end
			return count
		end,

		Update = function(self)
		end
	}
end


local function get_summary_table(systemid, sectorid, reset)
	if (not systemid) then return end

	summary_data[systemid] = summary_data[systemid] or {}

	if (sectorid) then
		if (reset) then
			summary_data[systemid][sectorid] = {}
		else
			summary_data[systemid][sectorid] = summary_data[systemid][sectorid] or {}
		end
		return summary_data[systemid][sectorid]
	else
		if (reset) then
			summary_data[systemid] = {}
		end
		return summary_data[systemid]
	end
end

function T.IsAsteroid()
	return T.RoidNames[GetTargetInfo()]
end

function T.GetSectorSummary(sectorid, refresh)
	if (not sectorid) then return {} end

	-- Return an empty summary if there is nothing to do
	local sector_data = T.GetData(GetSystemID(sectorid), sectorid)
	if (not next(sector_data)) then return {} end

	-- Refresh the sector summary if specified to or if there have been new roids scanned for the sector
	local summary = get_summary_table(GetSystemID(sectorid), sectorid, (refresh or T.pending[sectorid]))
	T.pending:Clear(sectorid)

	if (not next(summary)) then

		-- Look at all of the roids in the sector and count how many have each type of ore and record the highest percent per type
		for roidID, roid_data in pairs(sector_data) do
			for _, ore_type in ipairs(ore_reference[1]) do
				if (roid_data["ore"][ore_type]) then
					-- Make sure the ore table exists in the summary
					summary[ore_type] = summary[ore_type] or {}

					summary[ore_type]["count"] = (summary[ore_type]["count"] or 0) + 1

					-- Update the highest discovered ore percent in the sector for this ore
					if (roid_data["ore"][ore_type] > (summary[ore_type]["highest"] or 0)) then
						summary[ore_type]["highest"] = roid_data["ore"][ore_type]
					end
				end
			end

		end
	end
	return summary
end

function T.GetSystemSummary(systemid, refresh)
	if (not systemid) then return {} end


	-- Return an empty summary if there is nothing to do
	local system_data = T.GetData(systemid)
	if (not next(system_data)) then return {} end

	local summary = get_summary_table(systemid, nil, refresh)

	-- Always query the other sector summaries.  New data may have been acquired since the last time
	-- the function was run
	for sectorid, sector_data in pairs(system_data) do
		-- Explicitly request the sector summary, in case there's no entry in the system summary
		local sector_summary = T.GetSectorSummary(sectorid, refresh)
		for _, ore_type in ipairs(ore_reference[1]) do
			if (sector_summary[ore_type]) then
				-- Make sure the ore table exists in the summary
				summary[ore_type] = summary[ore_type] or {}

				summary[ore_type]["count"] = (summary[ore_type]["count"] or 0) + 1

				-- Update the highest discovered ore percent in the sector for this ore
				local ore_pct = sector_summary[ore_type] and sector_summary[ore_type]["highest"] or 0

				if (ore_pct > (summary[ore_type]["highest"] or 0)) then
					summary[ore_type]["highest"] = ore_pct
				end
			end
		end
	end

	return summary
end

function T.GetUniverseSummary(refresh)--systemid, sectorid, refresh)
	local data = T.GetData()

	for systemid in pairs(data) do
		T.GetSystemSummary(systemid, refresh)
	end
	return summary_data
end

function T.GetRoidCount(systemid, sectorid)
	if (not systemid) then return end

	local data = T.GetData(systemid, sectorid)

	if (sectorid) then
		data = {[sectorid] = data}
	end

	local count = 0
	for sectorid, sector_data in pairs(data) do
		for roidID in pairs(sector_data) do
			count = count + 1
		end
	end
	return count
end

function T.GetData(systemid, sectorid, create_table, delete_table)
	local data
	if (not systemid) then 
		data = asteroid_data
	else
		asteroid_data[systemid] = asteroid_data[systemid] or create_table and {}
		data = asteroid_data[systemid]
		
		if (data and sectorid) then
			if (delete_table) then
				data[sectorid] = nil
			else
				data[sectorid] = data[sectorid] or create_table and {}
			end
			data = data[sectorid]
		end
	end
	return data or {}
end

function T.GetStats()
	local data = T.GetData()
	local stats = {total = 0, [33] = 0}
	local sorted_systems = {}

	for i = 1, table.maxn(SystemNames) do
		stats[i] = 0
		table.insert(sorted_systems, {i, SystemNames[i]})
	end
	table.sort(sorted_systems, function(a, b) return a[2] < b[2] end)

	for _, ore in ipairs(ore_reference[1]) do stats[ore] = 0 end

	for systemid, system_data in pairs(data) do
		for _, sector_data in pairs(system_data) do
			for _, roid_data in pairs(sector_data) do
				stats["total"] = stats["total"] + 1
				stats[systemid] = stats[systemid] + 1

				for ore in pairs(roid_data["ore"]) do
					if (ore_reference[2][ore]) then -- Ran into some roid data with erroneous ore info
						stats[ore] = stats[ore] + 1
					end
				end
			end
		end
	end

	local stats_text = "Scanned Asteroids\n"..
	                   "Total: "..stats["total"].."\n\n"
	for _, system in ipairs(sorted_systems) do
		if (stats[system[1]] > 0) then
			stats_text = stats_text..system[2]..": "..stats[system[1]].."\n"
		end
	end
-- 	stats_text = stats_text.."Unknown: "..stats[33].."\n"..
	stats_text = stats_text.."\n".."Ore\n"

	for _, ore in ipairs(ore_reference[1]) do
		if (stats[ore] > 0) then
			stats_text = stats_text..ore_reference[2][ore]..": "..stats[ore].."\n"
		end
	end

	return stats_text
end

-- Outside-facing wrapper for T.GetData() to allow other plugins to... get data.
function roiddb.GetAsteroidData(systemID, sectorID, roidID)
	if (not (systemID or (systemID and sectorID))) then return end

	local data = T.GetData(systemID, sectorID)
	if (systemID and sectorID and roidID) then
		data = data[roidID]
	end
	return data
end

function T.DeleteRoidData(callback)
	-- Get the number of roids that are being deleted
	local count = 0
	for sectorID, delete_sector_data in pairs(T.DeleteRoidsList) do
		if (not next(delete_sector_data)) then
			delete_sector_data = T.GetData(GetSystemID(sectorID), sectorID)
		end

		for roidID in pairs(delete_sector_data) do
			count = count + 1
		end
	end

	local msg
	-- Function that is called when clicking "Yes" on the confirmation dialog
	local function delete_them()
		for sectorID, delete_sector_data in pairs(T.DeleteRoidsList) do
			local systemID = GetSystemID(sectorID)
			local data = T.GetData(systemID, sectorID) -- We want to be able to delete the sector from the system if there are no roids in it
			 -- not providing roid data.  only using the input to indicate a refresh is necessary
			 -- Doing this instead of trying to work T.DeleteRoids into the procedure, since
			 -- it was changed to force a completel refresh, anyway.
			T.pending:AddRoid(sectorID, "", nil)

			if (next(delete_sector_data)) then
				for roidID in pairs(delete_sector_data) do
					data[roidID] = nil
				end
			end

			-- If all roids were deleted, delete the sector.  Don't pollute the table
			if (not next(data) or not next(delete_sector_data)) then
				T.DeleteData(systemID, sectorID, true) -- delete the sector table, don't save.  We're saving below.
			end

			T.SaveData(systemID, true)

			if (type(callback) == "function") then callback() end
		end

		-- clean-up
		T.DeleteRoids = false
		T.DeleteRoidsList = {}
		HideDialog(msg)
		iup.Destroy(msg)
	end

	-- Function that is called when clicking "No" on the confirmation dialog
	local function abort_abort_we_almost_screwed_up()
-- 		T.DeleteRoids = false
-- 		T.DeleteRoidsList = {}
		HideDialog(msg)
		iup.Destroy(msg)
	end

	msg = msgdlgtemplate2(
		"You are about to delete "..count.." asteroid record(s).\n\nDo you wish to continue?",
		"Yes",
		delete_them,
		"No",
		abort_abort_we_almost_screwed_up
	)

	ShowDialog(msg)
end

function T.DeleteData(systemid, sectorid, dont_save)
	local data = T.GetData()

	if (systemid) then
		if (sectorid) then
			data[systemid][sectorid] = nil
			summary_data[systemid][sectorid] = nil

			if (not next(data[systemid])) then
				T.DeleteData(systemid)
				summary_data[systemid] = nil
			end
		else
			data[systemid] = nil
			summary_data[systemid] = nil
		end
	end

	if (not dont_save) then
		T.SaveData(systemid, true)
	end
end

function T.LoadData(systemid)
	if (new_data) then T.SaveData() end

	-- Local data
	local local_data_string = LoadSystemNotes(roiddb.notesid)
	local_data_string = local_data_string ~= "" and local_data_string or "pinned = {}"
	T.LocalData = unspickle(local_data_string)

	summary_data = {}
	--	asteroid_data = dofile("plugins/mdi-roiddb/roids.lua")
	asteroid_data = {} -- ?always initialize with empty table?
	for systemid = 1, T.HighestSystemID do
		local notes_data = LoadSystemNotes(roiddb.notesid + systemid)
		if (notes_data ~= "") then -- don't dirty the dataset if unnecesssary
			asteroid_data[systemid] = unspickle(notes_data)
		end
	end
	-- If asteroid dataset contains unknown system data, then modifiy the SystemNames to make references possible
	if (asteroid_data[33]) then
		SystemNames[33] = "Unknown"
		SystemNames["unknown"] = 33
		T.UI.unknownbtn.visible = "YES"
	end
end

function T.SaveData(systemid, force)
	if (new_data or force) then
		T.Save.timer:Kill()
		new_data = false
		local data

		if (systemid) then
			data = {[systemid] = T.GetData(systemid)}
		else
			data = T.GetData()
		end

		for systemid, system_data in pairs(data) do
			local notes = spickle_func(system_data)
			SaveSystemNotes(notes, roiddb.notesid + systemid)
		end
	end
end

function T.IsPinned(sectorID, roidID, onezero)
	local sector_data = T.LocalData.pinned[sectorID] or {}
	local retval = sector_data[roidID] and true or false
	if (onezero) then retval = retval and 1 or 0 end
	return retval
end

function T.TogglePin(sectorID, roidID)
	local sectortbl = T.LocalData.pinned[sectorID] or {}
	sectortbl[roidID] = not sectortbl[roidID] or nil -- I don't want to save false to the systemnotes
	T.LocalData.pinned[sectorID] = next(sectortbl) and sectortbl or nil
	SaveSystemNotes(spickle(T.LocalData), roiddb.notesid)
	return sectortbl[roidID]
end

function T.RoidsScanned(systemID, sectorID)
	local num_scanned = 0
	local est_total = 0
	for i in pairs(T.GetData(systemID or GetCurrentSystemid(), sectorID or GetCurrentSectorid())) do
		num_scanned = num_scanned + 1
		if (i > est_total) then est_total = i end
	end
	return num_scanned == est_total, num_scanned, est_total
end

T.Export = {}
function T.Export.SuccessMsg(source, to)
	T.MessageDlg:SetMessage(source.." data has been exported "..to..".", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end

function T.Export.RoidDB(notesid)
	local export = {}
	for systemid, system_data in pairs(T.GetData()) do
		export[systemid] = spickle_func(system_data)
	end

	SaveSystemNotes(spickle_func(export), notesid or -roiddb.notesid)
	T.Export.SuccessMsg("RoidDB", "to \"system"..-roiddb.notesid.."notes.txt\"")
end

-- This was a first attempt to export data to Prospector
-- The sector data would be read for the individual sectors when you were there,
-- but for some reason the data could not be searched, universe-wide.

-- function T.Export.Prospector()
-- 	local PR_SystemNotesPrefix = 91000000 -- hard-coded in case prospector isn't installed
--
-- -- sector_roid_data_table = {
-- -- 	MaxId = <number>,
-- -- 	RoidList = {
-- -- 		[<roid_id>] = {
-- -- 			[<ore_type>] = <number>,
-- -- 		}
-- -- 	}
-- -- }
--
-- 	-- Prospector saves each sector in it's own SystemNotes file.  This seems highly irresponsible to me.
--
-- 	local data = T.GetData()
--
-- 	for systemID, system_data in pairs(data) do
-- 		for sectorID, sector_data in pairs(system_data) do
--
-- 			local pr_sector_data = LoadSystemNotes(PR.SystemNotesPrefix + sectorID)
-- 			if (pr_sector_data == "") then
-- 				pr_sector_data = {MaxId = 0, RoidList = {}}
-- 			else
-- 				pr_sector_data = unspickle(pr_sector_data)
-- 			end
--
-- 			for roidID, roid_data in pairs(sector_data) do
--
-- 				-- Only add roid data if it's not currently present in the prospector dataset
-- 				if (not pr_sector_data[roidID]) then
-- 					pr_sector_data.RoidList[roidID] = {}
-- 					for ore_type, ore_pct in pairs(roid_data.ore) do
-- 						pr_sector_data.RoidList[roidID][ore_type] = ore_pct
-- 					end
-- 				end
-- 			end
--
-- 			pr_sector_data.MaxId = table.maxn(pr_sector_data.RoidList)
-- 			SaveSystemNotes(spickle(pr_sector_data), PR.SystemNotesPrefix + sectorID)
-- 		end
-- 	end
-- 	T.Export.SuccessMsg("Prospector 3", "\"settings/"..GetPlayerName().."\"")
-- end

T.Import = {}
function T.Import.NoDataMsg(source)
	T.MessageDlg:SetMessage("There is no "..source.." data to import.", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end

function T.Import.SuccessMsg(source, count)
	T.MessageDlg:SetMessage(count.." asteroids imported from "..source..".", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
	ShowDialog(T.MessageDlg)
end


function T.Import.RoidDB(notesid)
	local count = 0
	local status, temp
	local new_importer = true
	local import_data_text = LoadSystemNotes(notesid or -roiddb.notesid)
	local import_data = {}

	T.MessageDlg:SetMessage("There was an error importing RoidDB asteroid data.\nContact the plugin maintainer.", "OK", function(self) HideDialog(iup.GetDialog(self)) end)

	if (import_data_text:match("\n")) then
		-- SPICKLE2()
		if (import_data_text:match("\"\\")) then
			-- New format importer
			status, import_data = pcall(unspickle, import_data_text, true)
			if (not status) then
				ShowDialog(T.MessageDlg)
				return
			end

		else
			-- ---------------------------------------------------------
			-- Code for importing old-style spickle2() tables
			-- Retaining for now in case it becomes necessary
			count = 0 -- holds the systemid if within the range of 1 to n, terminated by a nil
			-- This pattern will match a system's data from a file created with spickle2()
			for system_data_text in import_data_text:gmatch("\n([%[{].-\r?\n},)") do
				-- because the data text being unspickled is inside {}, the inner table will need to
				-- be assigned to import_data, instead of the table returned by unspickle()
				-- index will need to be 1 if the systemid is in the contiguous range of 1 to n, terminated by a nil
				local systemid = tonumber(system_data_text:match("^%[(%d+)%]"))
				local index = systemid
				if (not systemid) then
					count = count + 1
					systemid = count
					index = 1
				end

				status, temp = pcall(unspickle, system_data_text, true)

				if (status) then
					import_data[systemid] = temp[index]
				else
					ShowDialog(T.MessageDlg)
					return
				end
			end
			new_importer = false
			-- ---------------------------------------------------------
		end

	else
		-- SPICKLE()
		status, import_data = pcall(unspickle, import_data_text, true)
		if (status) then
			local key, val = next(import_data)
			if (type(val) ~= "string") then
				new_importer = false
			end
		else
			ShowDialog(T.MessageDlg)
			return
		end
	end

	if (not next(import_data)) then
		T.Import.NoDataMsg("RoidDB")
		return
	end

	count = 0 -- holds the number of roids that have been imported
	for systemID, system_data in pairs(import_data) do
		if (new_importer) then
			status, system_data = pcall(unspickle, system_data, true)
			if (not status) then
				ShowDialog(T.MessageDlg)
				return
			end
		end
		for sectorID, sector_data in pairs(system_data) do
			local data = T.GetData(systemID, sectorID, true)

			for roidID, roidtbl in pairs(sector_data) do
				if (not data[roidID] or T.Scan.re) then
					data[roidID] = roidtbl
					count = count + 1
				end
			end
		end
	end

	T.Import.SuccessMsg("RoidDB", count)
	new_data = true
	T.SaveData()
end

function T.Import.Prospector()
	local PR_SystemNotesPrefix = 91000000 -- hard-coded in case prospector isn't installed
	local count = 0
	local gotsome = false
	-- Prospector saves each sector in it's own SystemNotes file.  This seems highly irresponsible to me.
	for sectorid = 1, ((T.HighestSystemID - 1) * 256 + (16 - 1) * 16 + 16) do -- sectorid = (s-1)*256 + (x-1)*16 + y
		local notes_data = LoadSystemNotes(PR_SystemNotesPrefix + sectorid)
		if (notes_data ~= "") then
			local data = T.GetData(GetSystemID(sectorid), sectorid, true)
			notes_data = unspickle(notes_data)
			gotsome = true
			for roidID, roidtbl in pairs(notes_data.RoidList) do
				if (not data[roidID] or T.Scan.re) then
					count = count + 1
					data[roidID] = {id = roidID, ore = {}}
					for ore_type, ore_pct in pairs(roidtbl) do
						data[roidID].ore[ore_type] = ore_pct
					end
				end
			end

		end
	end
	if (not gotsome) then
		T.Import.NoDataMsg("Prospector 3")
		return
	end
	T.Import.SuccessMsg("Prospector 3", count)
	new_data = true
	T.SaveData()
end

function T.Import.TargetLess()
	-- TargetLess data is stored by sector, with sector data being spickled separately from the universe
	local tldata = unspickle(LoadSystemNotes(741937)) -- unspickle the universe
	local count = 0
	if (not next(tldata)) then
		T.Import.NoDataMsg("TargetLess")
		return
	end

	for sectorid, sector_data in pairs(tldata) do
		sector_data = unspickle(sector_data)
		if (next(sector_data)) then
			local data = T.GetData(GetSystemID(sectorid), sectorid, true) -- RoidDB data is stored with sector data under the representative systemid
			for roidID, roidtbl in pairs(sector_data) do -- unspickle the sector data and operate over the roid data
				if (not data[roidID] or T.Scan.re) then -- only update roid data if it's a new roid
					count = count + 1
					data[roidID] = {id = roidID, ore = {}}
					for ore_type, ore_pct in pairs(roidtbl.ore) do
						data[roidID].ore[ore_reference[3][ore_type]] = tonumber(ore_pct)
					end
				end
			end
		end
	end
	T.Import.SuccessMsg("TargetLess", count)
	new_data = true
	T.SaveData()
end


function T.Import.OldRoidDB()
	local data = unspickle(LoadSystemNotes(764332))
	local count = 0

	if (not next(data)) then
		T.Import.NoDataMsg("Old RoidDB")
		return
	end

	for sectorid, sector_data in pairs(data) do
		local data = T.GetData(GetSystemID(sectorid), sectorid, true) -- RoidDB data is stored with sector data under the representative systemid
		for roidID, roidtbl in pairs(sector_data) do

			if (not data[roidID] or T.Scan.re) then
				count = count + 1
				data[roidID] = {id = roidID, ore = {}}
				for ore_type, ore_pct in pairs(roidtbl) do
					data[roidID].ore[ore_reference[3][ore_type]] = ore_pct
				end
			end

		end
	end
	T.Import.SuccessMsg("Old RoidDB", count)
	new_data = true
	T.SaveData()
end


T.help_text = [[

MDI-RoidDB commands:
/roiddb
/roiddbscan
/roiddbdeletesectordata
/roiddbstore

-----------------------------------
Filters
-----------------------------------
The filter options on the right can be used to restrict the results from a search, show only sectors with that ore on the system view, or show only asteroids with that ore in the sector view.  You can further filter the display by specifying a minimum percent of ore you are looking for.

-----------------------------------
Search
-----------------------------------
Search will return a list of systems or sectors with scanned asteroids that match the specified filter.

-----------------------------------
Universe View
-----------------------------------
From the universe view, select a system to view the sectors you have asteroids scanned in.  If you have scanned any asteroids in the Unknown system, there will be an additional button to select this system for viewing.

-----------------------------------
System View
-----------------------------------
The system view displays all of the sectors in the system that you have scanned asteroids in.  The highlighted systems will change to respect the filter you have selected.

-----------------------------------
Sector View
-----------------------------------
The sector view displays all of the asteroids you have scanned in the sector.  You can use the filters to only display asteroids with the specified ore.  The columns can be sorted by clicking on the headers at the top of the display.  The check boxes on the left side of the display can be used to pin the asteroid to the top of the display.  When an asteroid is selected, you can double-click it or press the target button to set your current target to that asteroid, assuming it is within range.  This action will close the display.

-----------------------------------
Auto-Scanner (/roiddbscan)
-----------------------------------
The auto-scanner will help find unscanned asteroids in the sector.  The auto-scanner has three different scanning modes:
]].."\t"..[[Continuous - This mode will continually scan for the cloest asteroid.
]].."\t"..[[Single Sweep - This mode will scan once for the closest asteroid.
]].."\t"..[[First In Range - This mode will stop scanning once it finds an asteroid in range.

The scanner will disengage automatically if all estimated asteroids in the sector have been scanned or if a non-asteroid is manually targeted.

-----------------------------------
Data Auto-Store
-----------------------------------
Asteroid data will be stored by default.  This can be disabled in options and will require manual input to store the data.  Use the command `/roiddbstore` to override the data store restriction.

-----------------------------------
Exporting Asteroid Records
-----------------------------------
You can export your asteroid database into a single file for simple backup or transfer.  This file can then be imported at a later time or on a different character.

-----------------------------------
Deleting Asteroid Records
-----------------------------------
You can delete asteroid records from your database by activating Delete Mode in Options.  You can delete individual asteroids from the Sector view or you can delete entire sectors from the System view.

In Sector view, you may select multiple asteroids to be deleted at once by tapping the asteroid row.
In System view, you may select multiple sectors to delete asteroids from my tapping the specific sectors.

Asteroids or sectors will turn red when selected.  The current sector will not turn red if selected, but the delete button will become active, to indicate your selection.

-----------------------------------
Other
-----------------------------------
If you take damage while the interface is open, it will automatically close so that you can respond to the source of damage.  This behavior can be disabled in Options.

-----------------------------------
MDI-TargetList
-----------------------------------
If you have MDI-TargetList v4.2+ installed, you will be able to view a list of asteroids (scanned or unscanned) from the HUD.

You can target an asteroid by using the commands `/targetN` (where N is 1-0), `/targetnext`, `/targetprev`, and `/targetthis` (used if the targeting controls are unlinked).

There are 3 filters you can use to control what asteroids are displayed:  Ore, Specific Ore, and Unscanned.  You can switch filters by using the command `/cyclefilter`.

The "Specific Ore" filter is a special one.  This is the filter to the left of "Ore".  This filter has multiple pages that each represent one type of ore.  Only asteroids with the specified ore will be shown.  You can change pages using the command `/cyclefilterpage`.  It will display the two-letter code describing the ore."

If you're on mobile, all of thees actions can be accomplished by tapping.
]]

-- -- -------------------------------
-- sectorid = (s-1)*256 + (x-1)*16 + y
-- -- -------------------------------

-- Roid Data
-- roid_data_table = {
-- 	[<system_id>] = {
-- 		[<sector_id>] = {
-- 			[<roid_id>] = {
-- 				id = <number>
-- 				ore = {
-- 					[<ore_type>] = <number>, -- <ore_type> should be 2-letter notation
-- 				},
-- 			},
-- 		},
-- 	}
-- }

-- Summary Data
-- 	[<systemid>] = {
-- 		[<sectorid>] = {
-- 			-- Sector ore summary
-- 			[[ore_type]] = {
-- 				count = <number>,
-- 				highest = <number>, -- (0 - 100)
-- 			},
-- 		},
-- 		-- System ore summary
-- 		[[ore_type]] = {
-- 			count = <number>,
-- 			highest = <number>, -- (0 - 100)
-- 		},
-- 	},
