local config_name = "MDI RoidDB"
declare("roiddb", {
	config_name = config_name,
	version = "1.1",
	notesid = 7643320, -- pinned roids
	--GetAsteroidData,
})


local T = {}
local function IsMobile(tf) return tf or Platform == "Android" or Platform == "iOS" end

T = {
	Events = {},
	FONT_SCALE = 1,
	Font = Font.H5 * 1, --(HUD_SCALE or 1),
	IMAGE_DIR = "plugins/mdi-roiddb/images/",
	disable_auto_close = gkini.ReadInt(config_name, "Disable Auto Close", 0) == 1,
	load_tab = gkini.ReadInt(config_name, "Load Tab", 1) == 1,
	use_mobile_layout = IsMobile() and gkini.ReadInt(config_name, "Use Mobile Layout", 1) == 1, -- Causes IsMobile() to return false if the value is true.  Only want to allow it to be changed if on a mobile device
	HighestSystemID = 33,
	LocalData = {
		pinned = {},
	},
	Scan = {
		AutoScanStartStop,
		GetScannerRange,
		GetSectorNodeID,
		Lock,
		LockFunction,
		PingTarget,
		ReTarget,
		ScannedRoids,
		TargetNextClosestRoid,
		UnscannedRoids,
		-- -----------------
		dont_store = gkini.ReadInt(config_name, "Don't Store", 0) == 1, -- Don't store scanned roids automatically
		seek_mode = gkini.ReadInt(config_name, "Seek Mode", 2), -- [1] Continuous, [2] Single Sweep, [3] First In Range; Default: 2
		lock = false,
		ningAuto = false, -- Use if auto-scan is in progress.  Referenced in event TARGET_SCANNED to get the next target
		ping_timer = Timer(),
		re = gkini.ReadInt(config_name, "Rescan Asteroids", 0) == 1, -- rescan asteroids
		range = -1,
		interval = 5000,
		roid_list = {},
		store_ok = false, -- override dont_store
		timer = Timer(), -- used by T.Scan.ReTarget() and T.Scan.TargetnextClosestRoid and when the matrix is being populated
		num_scanned = 0, -- The number of roids scanned in the current sector
		est_total = 0, -- the estimated total of roids in the current sector
	},
	-- RoidData
	Save = {
		timer = Timer{},
		interval = gkini.ReadInt(config_name, "Save Interval", 60) * 1000,
	},
	Tab = {}, -- MDI-TargetList tab integration.  Defined in targetlist.lua
	UI = {},
	MessageDlg = msgdlgtemplate1(),
	IsMobile = IsMobile,
}
roiddb.T = T

console_print("MDI-RoidDB loaded (v"..roiddb.version..").")

function T:SaveSettings()
	gkini.WriteInt(config_name, "Don't Store", T.Scan.dont_store and 1 or 0)
	gkini.WriteInt(config_name, "Save Interval", T.Save.interval / 1000)
	gkini.WriteInt(config_name, "Rescan Asteroids", T.Scan.re and 1 or 0)
	gkini.WriteInt(config_name, "Disable Auto Close", T.disable_auto_close and 1 or 0)
	gkini.WriteInt(config_name, "Load Tab", T.load_tab and 1 or 0)
	if (T.IsMobile()) then
		gkini.WriteInt(config_name, "Use Mobile Layout", T.use_mobile_layout and 1 or 0)
	end
	gkini.WriteInt(config_name, "Seek Mode", T.Scan.seek_mode or 2)
end

local function check_number(num, Lval, Uval, dp)
	-- verify a number is within the given range
	-- <num> the number to check
	-- <Lval>, <Uval>: Lower and Upper bounds of the number range; if missing, compares the number against itself
	-- [dp]:  how many decimal places to consider
	num = tonumber(num)
	Lval = tonumber(Lval) or num
	Uval = tonumber(Uval) or num
	if (num and Lval <= num and Uval >= num) then
		if (dp) then
			local decimals = tostring(num):match("%.(%d+)") or "0"
			if (decimals and decimals:len() <= dp) then
				return num -- entered number has no more decimal places than specified
			else
				return false -- too many decimal places
			end
		else
			return num -- non-decimal number is within range
		end
	end
	return false  -- you done fucked up
end

local function loadluafile(...)
    local f, err = loadfile((...))
    assert(f and type(f) == "function", err)
    return f(select(2, ...))
end

loadluafile("spickle2.lua")
loadluafile("data.lua", T)
loadluafile("scanning.lua", T)
loadluafile("events.lua", T)
loadluafile("ui.lua", T, loadluafile, check_number)
if (T.load_tab) then
	loadluafile("targetlist_tab.lua", T)
end


RegisterUserCommand("roiddb", function() ShowDialog(T.UI.maindlg) end)
RegisterUserCommand("roiddbscan", T.Scan.AutoScanStartStop)
RegisterUserCommand("roiddbdeletesectordata", function()
	T.DeleteData(GetCurrentSystemid(), GetCurrentSectorid())
	print(string.format("%sDeleting asteroid data for %s\127o", "\127FF0000", ShortLocationStr(GetCurrentSectorid())))
end)
RegisterUserCommand("roiddbstore", function()
	if (not T.Scan.store_ok) then
		T.Scan.store_ok = true
		T.Scan.LockFunction(T.Scan.ReTarget)
	end
end)
