local T, check_number = ...

local number_ranges = {
	{low = 0, high = 255,},
	{low = 0.01, high = 2, dp = 2},
	{low = 0, high = gkinterface.GetXResolution()},
	{low = 0, high = gkinterface.GetYResolution()},
}

local function get_control_width(generator, str, font)
	local c = iup[generator]{[generator == "text" and "value" or "title"] = str, font = font, expand = "NO"}
	iup.Map(c)
	local w = tonumber(c.w)
	iup.Destroy(c)
	return w
end

local function get_control_dimensions(size)
	local c = iup.canvas{size = size, expand = "NO"}
	iup.Map(c)
	local w, h = tonumber(c.w), tonumber(c.h)
	iup.Destroy(c)
	return w, h
end

local function check_decimal_prefix(str, dp)
	local lead = str:match("^0?%.?$")
	if (lead == "") then lead = nil end
	local trail = str:match("^0?%.(0-)$")
	return trail and trail:len() < dp or lead
end


-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                     Export/Import UI                      ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###

local function BuildExportImportUI(exporting)
	local dlg, container
	local exportimport_radio
	local exportimport_options
	local cancelbtn, okbtn

-- ### ######################################################### ###
-- ### Export/Import UI - Layout                                 ###
-- ### ######################################################### ###



	cancelbtn        = iup.stationbutton{title = "Cancel"}
	okbtn            = iup.stationbutton{title = "OK"}


	local prefix = exporting and "Export" or "Import"
	local direction = exporting and " to " or " from "
	exportimport_options = {
		iup.stationradio{title = prefix.." RoidDB Data",            font = T.Font},
		iup.stationradio{title = prefix..direction.."Prospector 3", font = T.Font, active = "NO"},
		iup.stationradio{title = prefix..direction.."TargetLess",   font = T.Font, active = "NO"},
		iup.stationradio{title = prefix..direction.."Old RoidDB",   font = T.Font, active = "NO"},
		gap = 5,
	}
	exportimport_radio = iup.radio{iup.vbox(exportimport_options)}
	exportimport_radio.value = exportimport_options[1]

	local gap = 10
	container = iup.hbox{
		iup.vbox{
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						iup.label{title = prefix.."..."},
						iup.fill{expand = "YES"},
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.vbox{
						iup.hbox{
							exportimport_radio,
							gap = gap,
						},
						-- ------------------
						gap = gap,
						expand = "NO",
						margin = "5x5",
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						--resetbtn,
						iup.fill{},
						cancelbtn,
						--applybtn,
						okbtn,
						gap = gap,
					},
				},
			},
			gap = gap,
		},
		expand = "NO",
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				container,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = "YES",
		fullscreen = "YES",
		defaultesc = cancelbtn,
		bgcolor = "0 0 0 64 *"
	}

-- ### ######################################################### ###
-- ### Export/Import UI - Callbacks                              ###
-- ### ######################################################### ###

	function okbtn:action()
		if (exportimport_radio.value == exportimport_options[1]) then -- RoidDB
			if (exporting) then
				T.Export.RoidDB()
			else
				T.Import.RoidDB()
				T.UI.matrix:refresh()
			end

		elseif (exportimport_radio.value == exportimport_options[2]) then -- Prospector
			if (exporting) then
				T.Export.Prospector()
			else
				T.Import.Prospector()
				T.UI.matrix:refresh()
			end

		elseif (exportimport_radio.value == exportimport_options[3]) then -- TargetLess
			if (exporting) then
				T.Export.TargetLess()
			else
				T.Import.TargetLess()
				T.UI.matrix:refresh()
			end

		elseif (exportimport_radio.value == exportimport_options[4]) then -- Old RoidDB
			if (not exporting) then
				T.Import.OldRoidDB()
				T.UI.matrix:refresh()
			end
		end

		if (not exporting) then
			-- If Importing then refresh the entire summary.  This is necessary for Search to operate properly
			T.GetUniverseSummary(true)
		end

		cancelbtn:action()
	end

	function cancelbtn:action()
		HideDialog(iup.GetDialog(self))
	end

	function dlg:show_cb()
		if (not exporting) then
			exportimport_options[2].active = "YES" -- enable Prospector for import only
			exportimport_options[3].active = "YES" -- enable TargetLess for import only
			exportimport_options[4].active = "YES" -- enable Old RoidDB for import only
		end
	end

	function dlg:close_cb()
		iup.Destroy(dlg)
	end

	iup.Map(dlg)

	return dlg

end

-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                         Help                              ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###
function BuildHelpUI(stats)
	local dlg
	local content
	local multi
	local closebtn
	local help_container, content_container

-- ### ######################################################### ###
-- ### Help - Interface Layout                                   ###
-- ### ######################################################### ###

	local gap = 10
	multi = iup.multiline{value = stats and T.GetStats() or T.help_text, size = "TWOTHIRDxTWOTHIRD", readonly = "YES"}

	closebtn = iup.stationbutton{title = "Close"}

	content_container = iup.vbox{
		multi,
	}

	help_container = iup.hbox{
		iup.vbox{
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						iup.label{title = stats and "Statistics" or "Help"},
						iup.fill{expand = "YES"},
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.vbox{
						iup.hbox{
							content_container,
							gap = gap,
						},
						-- ------------------
						gap = gap,
						--expand = "NO",
						margin = "5x5",
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						--resetbtn,
						iup.fill{},
						closebtn,
						gap = gap,
					},
				},
			},
			gap = gap,
		},
		expand = "NO",
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				help_container,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = "YES",
		fullscreen = "YES",
		defaultesc = closebtn,
		bgcolor = "0 0 0 64 *"
	}



-- ### ######################################################### ###
-- ### Help - Interface Callbacks & Supporting Code              ###
-- ### ######################################################### ###

	function closebtn:action()
		local dlg =iup.GetDialog(self)
		HideDialog(dlg)
		iup.Destroy(dlg)
	end

	dlg:map()
	return dlg
end




-- ### ######################################################### ###
-- ### ######################################################### ###
-- ###                                                           ###
-- ###                       Options UI                          ###
-- ###                                                           ###
-- ### ######################################################### ###
-- ### ######################################################### ###

function T:BuildOptionsUI()

	local dlg, options_container, content_container
	local save_interval
	local rescan, disable_auto_close, load_tab, dont_store, use_PC_layout
	local seek_mode, continuous, single_sweep, first_in_range, seek_mode_items
	local delete, deletebtn
	local exportbtn, importbtn, statsbtn
	local cancelbtn, applybtn, okbtn, resetbtn, helpbtn


-- ### ######################################################### ###
-- ### Options UI - Interface Layout                             ###
-- ### ######################################################### ###
	local text_width   = get_control_width("text", "000000")
	local dialog_gap = 10
	local content_gap = 10

	save_interval      = iup.text{size = text_width}

	rescan             = iup.stationtoggle{title = "Rescan Asteroids"}
	dont_store         = iup.stationtoggle{title = "Don't Auto-Store Asteroid Data"}
	disable_auto_close = iup.stationtoggle{title = "Disable Auto-Close on Damage"}
	load_tab           = iup.stationtoggle{title = "Load MDI-TargetList Tab"}
	use_PC_layout      = iup.stationtoggle{title = "Use PC Layout", visible = T.IsMobile() and "YES" or "NO"}

	continuous         = iup.stationradio{title = "Continuous", expand = "YES"}
	single_sweep       = iup.stationradio{title = "Single Sweep"}
	first_in_range     = iup.stationradio{title = "First In Range"}
	seek_mode_items    = {continuous, single_sweep, first_in_range, gap = content_gap}
	for i, c in ipairs(seek_mode_items) do c.position = i end
	seek_mode          = iup.radio{iup.vbox(seek_mode_items)}

	delete             = iup.stationtoggle{title = "Delete Mode"}

	exportbtn          = iup.stationbutton{title = "Export Data"}
	importbtn          = iup.stationbutton{title = "Import Data"}

	helpbtn            = iup.stationbutton{title = "Help"}
	statsbtn           = iup.stationbutton{title = "Statistics"}

	cancelbtn          = iup.stationbutton{title = "Cancel"}
	applybtn           = iup.stationbutton{title = "Apply", active = "NO"}
	okbtn              = iup.stationbutton{title = "OK"}

	content_container = iup.vbox{
		iup.hbox{
			iup.label{title = "Save Interval  "},
			save_interval,
			iup.label{title = "s"},
			iup.fill{expand = "YES"}

		},
		iup.hbox{
			iup.vbox{
				rescan,
				dont_store,
				disable_auto_close,
				load_tab,
				use_PC_layout,
				gap = content_gap,
			},
			iup.fill{size = content_gap * 2},
			iup.vbox{
				iup.pdarootframe{
					iup.vbox{
						iup.label{title = "Scanning Mode"},
						seek_mode,
						expand = "NO"
					},
				},
				gap = content_gap,
			},
		},
		iup.hbox{delete, iup.fill{}, deletebtn, iup.fill{}, alignment = "ACENTER"},
		iup.fill{size = content_gap},
		iup.hbox{
			iup.fill{},
			iup.vbox{
				exportbtn,
				importbtn,
				alignment = "ACENTER",
				gap = content_gap,
			},
			iup.fill{},
			iup.vbox{
				statsbtn,
				helpbtn,
				alignment = "ACENTER",
				gap = content_gap,
			},
			iup.fill{}
		},
		gap = content_gap,
	}

	options_container = iup.hbox{
		iup.vbox{
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						iup.label{title = "Options"},
						iup.fill{expand = "YES"},
						iup.label{title = "v"..(roiddb.version or "")},
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.vbox{
						iup.hbox{
							content_container,
							gap = dialog_gap,
						},
						-- ------------------
						gap = dialog_gap,
						--expand = "NO",
						margin = "5x5",
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						--resetbtn,
						iup.fill{},
						cancelbtn,
						applybtn,
						okbtn,
						gap = dialog_gap,
					},
				},
			},
			gap = dialog_gap,
		},
		expand = "NO",
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				options_container,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = "YES",
		fullscreen = "YES",
		defaultesc = cancelbtn,
		bgcolor = "0 0 0 64 *"
	}


-- ### ######################################################### ###
-- ### Options UI - Callbacks                                    ###
-- ### ######################################################### ###

	local apply_settings, load_settings, reset_settings, merge_color_parts, make_dirty, is_mobile_input_good, use_PC_changed

	local function num_text_action(Lval, Uval, dp, action)
		-- Action function that will only allow number-based entries within a given range
		-- NOTE:  Doesn't support negative values
		-- <Lval>, <Uval>:  Lower and Upper bounds in the number range
		-- [dp]: how many decimal places to consider, or the [action] function
		-- [action]:  function to be called upon success
		if (type(dp) == "function") then
			action = dp
			dp = nil
		end

		return function(self, c, after)
			local num = tonumber(after)
			if (c == 8  or c == iup.K_BS or -- backspace
				c == 9  or -- tab
				c == 27 or -- escape
				c == 13 or -- enter
				dp and (c == 46 or c == 48) and check_decimal_prefix(after, dp) or -- 46: .   48: 0
				check_number(num, Lval, Uval, dp)
			) then
				if (not dp and c == 46) then return iup.IGNORE end -- if there's no decimal allowed and they try to enter a decimal point
				if (type(action) == "function") then action(c, num) end
				make_dirty(is_mobile_input_good)
				return iup.DEFAULT
			else
				return iup.IGNORE
			end
		end
	end

	save_interval.action = num_text_action(1)

	function rescan:action(v) make_dirty() end

	function dont_store:action(v) make_dirty() end

	function disable_auto_close:action(v) make_dirty() end

	function load_tab:action(v) make_dirty() end

	function use_PC_layout:action(v)
		-- Track the original state of use_PC_layout when the toggle is pressed
		if (not use_PC_changed) then
			if (v == 1) then
				use_PC_changed = "OFF"
			else
				use_PC_changed = "ON"
			end
		end
		make_dirty()
	end


	-- Seek Mode Toggles
	for _, t in ipairs(seek_mode_items) do
		function t:action(v) make_dirty() end
	end

	function delete:action(v)
		make_dirty()
	end

	function exportbtn:action()
		ShowDialog(BuildExportImportUI(true))
	end

	function importbtn:action()
		ShowDialog(BuildExportImportUI())
	end

	function statsbtn:action()
		ShowDialog(BuildHelpUI(true))
	end

	function helpbtn:action()
		ShowDialog(BuildHelpUI())
	end

	function okbtn:action()
		if (applybtn.active == "YES") then
			if (not applybtn:action()) then return end
		end
		--iup.Close(dlg)
		--HideDialog(iup.GetDialog(self))
		cancelbtn:action()
	end

	function applybtn:action()
		return apply_settings()
	end

	function cancelbtn:action()
		use_PC_changed = nil
		applybtn.active = "NO"
		HideDialog(iup.GetDialog(self))
	end

-- 	function resetbtn:action()
-- 		reset_settings()
-- 		make_dirty()
-- 	end

	function dlg:show_cb()
		-- If returning from the positioner, don't reload the options.
		load_settings()
	end

	function dlg:close_cb()
	end

-- --------------------------------------------------

	local function set_text_value(ctrl, val)
		ctrl.value = val
		ctrl.fgcolor = "255 255 255"
	end

	load_settings = function()
		set_text_value(save_interval, T.Save.interval / 1000)
		rescan.value = T.Scan.re and "ON" or "OFF"
		disable_auto_close.value = T.disable_auto_close and "ON" or "OFF"
		dont_store.value = T.Scan.dont_store and "ON" or "OFF"
		load_tab.value = T.load_tab and "ON" or "OFF"
		use_PC_layout.value = T.use_mobile_layout and "OFF" or "ON"
		seek_mode.value = seek_mode_items[T.Scan.seek_mode]
		delete.value = T.DeleteRoids and "ON" or "OFF"
	end

	reset_settings = function()
		set_text_value(save_interval, T.Save.interval / 1000)
		rescan.value = "OFF"
		disable_auto_close.value = "OFF"
		dont_store = "OFF"
		load_tab.value = "ON"
		use_PC_changed = T.IsMobile() and use_PC_layout.value
		use_PC_layout.value = T.IsMobile() and "OFF" or "ON"
		seek_mode.value = single_sweep
		make_dirty()
	end

	apply_settings = function()
		if (not is_mobile_input_good()) then return false end
		T.Save.interval = save_interval.value * 1000
		T.Scan.re = rescan.value == "ON"
		T.disable_auto_close = disable_auto_close.value == "ON"
		T.Scan.dont_store = dont_store.value == "ON"
		T.load_tab = load_tab.value == "ON"
		T.use_mobile_layout = use_PC_layout.value == "OFF"
		T.Scan.seek_mode = tonumber(seek_mode.value.position)

		T.DeleteRoids = delete.value == "ON"

		if (T.DeleteRoids) then
			T.UI.delete_button_container:show()
		else
			T.UI.delete_cancel:action()
		end

		T:SaveSettings()
		applybtn.active = "NO"

		if (use_PC_changed and use_PC_changed ~= use_PC_layout.value) then
			ReloadInterface()
		end
		use_PC_changed = nil

		return true
	end

	make_dirty = function(action)
		applybtn.active = "YES"
		if (type(action) == "function") then
			action()
		end
	end

	merge_color_parts = function(r, g, b, a, blend)
		r = r ~= "" and r or 0
		g = g ~= "" and g or 0
		b = b ~= "" and b or 0
		a = a ~= "" and a or 0
		return string.format("%s %s %s %s %s",r, g, b, a, blend)
	end

	local function mobile_check_control(ctrl, l, h, d)
		if (check_number(ctrl.value, l, h, d) or ctrl.value == "") then
			ctrl.fgcolor = "255 255 255"
		else
			ctrl.fgcolor = "255 0 0"
			return false
		end
		return true
	end

	is_mobile_input_good = function()
		local ok_to_apply = true
		ok_to_apply = mobile_check_control(save_interval, 1) and ok_to_apply
		return ok_to_apply
	end


	iup.Map(dlg)
	T.UI.optionsdlg = dlg
	T.UI.delete_toggle = delete
end

T:BuildOptionsUI()


